`CGHseg` <-
function(y, chrV = rep(1, length(y)), Km = 20, S = -0.5,
                   verbose = FALSE)  {
  nona <- which(!is.na(y))
  y.nona <- y[nona]
  chrV.nona <- chrV[nona]

  output <- NULL
  unique.chrs <- unique(chrV.nona)
  for(chr in unique.chrs)  {
    if(verbose)
      cat("Processing chromosome ", chr, "\n", sep = "")

    idxchr <- which(chrV.nona == chr)
    Y1 <- y.nona[idxchr]
    N <- length(Y1)

    if( Km > floor(N/4) )  {
      Km <- floor(N/4)
      if(verbose)
        cat("\tNot enough data, Km set to ", Km, "\n", sep = "")
    }
    res <- .C("CGHseg3_th",
              N = as.double(N),
              Y1 = as.double(Y1),
              Km = as.integer(Km),
              J = double(Km),
              th = integer(Km*Km))
    dim(res$th) <- c(Km,Km)

    ## Adaptive penalty selection of # segments
    Jtild <- (res$J[Km] - res$J) / (res$J[Km] - res$J[1]) * (Km - 1) + 1
    temp <- which(diff(diff(Jtild)) > -S)
    if(length(temp) > 0)
      Kh <- max(temp) + 1
    else
      Kh <- 1
    
    ## Summarize breakpoint data
    eidx.nona <- res$th[Kh,1:Kh]
    sidx.nona <- c(1, eidx.nona[-Kh] + 1)

    meanseg <- numeric(Kh)
    for(i in 1:Kh)
      meanseg[i] <- mean(Y1[sidx.nona[i]:eidx.nona[i]])

    sidx <- nona[idxchr[sidx.nona]]
    eidx <- nona[idxchr[eidx.nona]]

    tab.chr <- data.frame(Chromosome = rep(chr,Kh),
                          StartIdx = sidx,
                          EndIdx = eidx,
                          Size = eidx-sidx+1,
                          NumFiniteLogRatios = eidx.nona-sidx.nona+1,
                          Mean = meanseg)
    output <- rbind(output, tab.chr)
  }
  return(output)
}

