`calcBioHMM` <-
function(y, ys = NULL,
                       chrs = rep.int(1, length(y)),
                       spos = 2*(1:length(y)) - 1,
                       epos = 2*(1:length(y)),
                       ...)  {
  if(is.null(ys))
    ys <- y

  input <- list(M.observed = matrix(ys, ncol = 1),
                genes = data.frame(Chr = chrs, Position = (spos+epos)/2), design = 1)
  ## run.nelder can sometimes throw an error at data it does not like
  try(stateV <- runBioHMM(input, ...)$state[,1])

  if(exists("stateV"))  {
    yseg <- numeric(length(y))
    for(chr in unique(chrs))  {
      idx.chr <- which(chrs == chr)
      state.chr <- stateV[idx.chr]
      for(state in unique(state.chr))  {
        idx.state.chr <- state.chr == state
        yseg[idx.chr[idx.state.chr]] <- mean(y[idx.chr[idx.state.chr]], na.rm = TRUE)
      }
    }

    yseg[is.na(y)] <- NA
    attr(yseg, "dataType") <- "SLR"
    return(yseg)
  }else  {
    cat("Error triggered within BioHMM algorithm!\n")
    tmp <- rep(NA, length(y))
    attr(tmp, "dataType") <- "error"
    return(tmp)
  }
}

