`cghImage` <-
function(Z, resType, zlim, colV, lwd = 1,
                     drawBoundaries = TRUE, dpi = 100)  {
  ## The slope and intercepts here are used to link log-ratios to their
  ## appropriate color in the color vector
  n <- length(colV)
  slope <- (n-1)/(zlim[2]-zlim[1])
  intercept <- 1 - slope*zlim[1]

  nr <- nrow(Z)
  nc <- ncol(Z)

  ## How many pixels by width are there available in the plot?
  tmp <- par()
  npix <- round((tmp$fin[1] - tmp$mai[2] - tmp$mai[4])*dpi)

  ## We define a function accessible only to cghImage()
  posnegmax <- function(x)  {
    ans <- x[which.max(abs(x))]
    if(length(ans) == 1)
      return(ans)
    else
      return(NA)
  }
  
  ## Take +/- maximum of data points if the number of data points exceeds
  ## the number of pixels by width.  This is done for two reasons:
  ## (a) It speeds up rendering considerably for very large data sets
  ## (b) It removes the bias of showing only the last data value per pixel
  ##     when drawing large data sets
  if(npix < nr)  {
    x <- round(seq(1, nr, length.out = npix+1))
    sx <- x[-(npix+1)]
    ex <- c(sx[-1] - 1, nr)
    locp <- 1:npix
    ftmp <- as.factor(rep(locp, times = (ex-sx+1)))

    ## ylim was defined so that data from the first column of Z will
    ## be plotted at the top of the heatmap rather than at the bottom
    ## of the heatmap.
    plot(NA, xlim = c(0.5, npix+0.5), ylim = c(nc+0.5, 0.5),
         axes = FALSE, xlab = "", ylab = "", main = "",
         xaxs = "i", yaxs = "i")
    for(j in 1:nc)  {
      if(resType[j] == "SLR" | resType[j] == "OLR")  {
        ## For each pixel, we use the log-ratio with the largest amplitude.
        ## We could have taken the mean log-ratio, but the resulting
        ## heatmap looks quiet and obscures the existence of
        ## focal aberrations.
        y <- tapply(Z[,j], ftmp, posnegmax)
        idx <- round(slope*y+intercept)
        idx[idx > n] <- n
        idx[idx < 1] <- 1
        segments(x0=locp, x1=locp, y0=j-0.5, y1=j+0.5, col = colV[idx])
      }else if(resType[j] == "error")  {
        text(x = npix/2, y = j, label = "Algorithm Error!")
      }
    }
  }else  {
    plot(NA, xlim = c(0.5, nr+0.5), ylim = c(nc+0.5, 0.5),
         axes = FALSE, xlab = "", ylab = "", main = "",
         xaxs = "i", yaxs = "i")
    locp <- 1:nr
    for(j in 1:nc)  {
      if(resType[j] == "SLR" | resType[j] == "OLR")  {
        y <- Z[,j]
        idx <- round(slope*y+intercept)
        idx[idx > n] <- n
        idx[idx < 1] <- 1
        rect(xleft = locp-0.5, xright = locp+0.5,
             ytop = j-0.5, ybottom = j+0.5,
             col = colV[idx], border = colV[idx])
      }else if(resType[j] == "error")  {
        text(x = nr/2, y = j, label = "Algorithm Error!")
      }
    }
  }

  if(drawBoundaries)  {
    abline(h = 0.5+1:(ncol(Z)-1), col = "black", lwd = lwd)
    box()
  }
  
}

