`checkUploadFileFormat` <-
function (tab,genomeType="HG18", tempDir=getwd(),resultDir="CGHResults",Threshold =0.2,minLR=-2,maxLR=2,apList)
{

    uDir <- tempDir
    tDir <- file.path(tempDir, resultDir)
    maxProbesChrom <- 10000
     cbFN<-genomeType

 

  ## Begin log
  cat("----------------------------------------\n")
  cat("Checking Array\n")
 

  ## Read in relevant R scripts
  #source(file.path(rScriptsDir, "sortChrs.R"))
 # source(file.path(rScriptsDir, "FormatDF.R"))

  ## The uploaded file must be in tab-delimited format and contain
  ## at least the following column names: ProbeID, Chromosome, Position, LogRatio

  ## Check file for proper headers
  ## (scan first 100 lines for header, assuming users might use the
  ##  "#" sign to annotate their text file)

  ## We also filter out "empty" lines, which is possible with user error
   reqdHeaders <- c("ProbeID", "Chromosome", "Position", "LogRatio")

  if(all(reqdHeaders %in% colnames(tab)))  {

    ## Remove "chr" from the Chromosome column, if any
    tab$Chromosome <- sub("^[chr|CHR|chr0|CHR0]*", "", tab$Chromosome)

    unique.chrs <- unique(tab$Chrom)

    ## Are the data sorted by chromosome?
    ## Within each chromosome, are the data sorted by position?
    sortedByChrom <- TRUE
    sortedByPos <- TRUE

    for(chr in unique.chrs)  {
      idx.chr <- which(tab$Chrom == chr)
      if(any(diff(idx.chr) != 1))
        sortedByChrom <- FALSE

      if(any(diff(tab$Position[idx.chr]) < 0))  {
        sortedByPos <- FALSE
        break
      }
    }

    ## If the data are not in order, sort the data
    if(!sortedByChrom | !sortedByPos)  {
      if(!sortedByChrom)
        unique.chrs <- sortChrs(unique.chrs)

      tab.sorted <- NULL
      for(chr in unique.chrs)  {
        tab.chr <- tab[tab$Chrom == chr, ]
        tab.chr <- tab.chr[order(tab.chr$Position, tab.chr$ProbeID), ]
        tab.sorted <- rbind(tab.sorted, tab.chr)
      }

      tab <- tab.sorted
    }

    ## Check if there are probes with duplicate positions within each chromosome
    idxDuplicatePos <- numeric(0)
    for(chr in unique.chrs)  {
      idx.chr <- which(tab$Chrom == chr)
      tmp <- diff(tab$Position[idx.chr])
      if(any(tmp == 0))
        idxDuplicatePos <- c(idxDuplicatePos, idx.chr[tmp == 0] + 1)
    }

    ## Are there redundant probe IDs?
    ## If so, relabel the redundant probe IDs
    redundantPIDs <- unique(tab$ProbeID[duplicated(tab$ProbeID)])
    nrPIDs <- sum(duplicated(tab$ProbeID))
    if(length(redundantPIDs) > 0)  {
      tmp <- tab$ProbeID
      for(r in redundantPIDs)  {
        idx <- which(tmp == r)
        tmp[idx] <- paste(r, 1:length(idx), sep = "_duplicate_")
      }
      tab$ProbeID <- tmp
    }

    ## How many probes are there per chromosome?  Of those, how many of them
    ## have finite values
    totalProbesPerChrom <- table(tab$Chrom)[unique.chrs]
    finiteProbesPerChrom <- tapply(is.finite(tab$LogRatio), as.factor(tab$Chrom), sum)[unique.chrs]
    probeCountSummary <- data.frame(unique.chrs, totalProbesPerChrom, finiteProbesPerChrom)
    colnames(probeCountSummary) <- c("Chromosome", "Probes", "Probes with Finite Values")

   
    ## If there are more than 'maxProbesChrom' number of probes in at least one of the chromosomes,
    ## and the user has selected to use a memory hungry algorithm such as CGHseg or cghFLasso,
    ## tell the user that the server will divide up the data for those algorithms.
    ##
    ## Note: 'maxProbesChrom' is defined in settings.R
    methodsDivideChrom <- NULL
    if(any(totalProbesPerChrom > maxProbesChrom))  {
      if("calcPicard" %in% names(apList))
        methodsDivideChrom <- "calcPicard"
      if("calcFL" %in% names(apList))
        methodsDivideChrom <- c(methodsDivideChrom, "calcFL")
    }

    ## If there are probes within a chromosome with duplicate positions, do not run
    ## BioHMM** or FASeg.  With such data, FASeg goes into an infinite loop, and BioHMM
    ## will throw errors from run.nelder() when estimating HMM parameters.
    ##
    ## ** Only if BioHMM is set to consider probe distances
    methodsIgnored <- NULL
    if(length(idxDuplicatePos) > 0)  {
      if("calcBioHMM" %in% names(apList))  {
        if(apList$calcBioHMM$useCloneDists == TRUE)
          methodsIgnored <- "calcBioHMM"
      }

      if("calcFASeg" %in% names(apList))
        methodsIgnored <- c(methodsIgnored, "calcFASeg")
    }

    if(!is.null(methodsIgnored))
      apList <- apList[-match(methodsIgnored, names(apList))]

    if(length(apList) > 0)  {
      #save(apList, Threshold, minLR, maxLR, genomeType, tab, methodsIgnored, methodsDivideChrom, cbFN,          file = file.path(tDir, "inputDF.RData"))
      continueAnalysis <- TRUE
    }else  {
      ## Do not continue with aCGH analysis
      continueAnalysis <- FALSE
    }

    ## Close log
  
  
    

     cat("Input array information:\nNumber of rows:", nrow(tab), "\nNumber of columns:", ncol(tab), "\n",sep="")
    cat("Number of probes with undefined values:", sum(is.na(tab$LogRatio)), "\n",sep="")

      if(length(redundantPIDs) > 0)  {
      cat("Warning: the uploaded data have", nrPIDs, "probes with duplicate identifiers ")
      cat("in the ProbeID column.  Those probes have been relabeled.\n")
    }

    if(!sortedByChrom | !sortedByPos)  {
      cat("Warning: the uploaded data were not sorted by chromosome and position! \n ")
      cat("The data have been rearranged.\n")
    }

    if(length(idxDuplicatePos) > 0)  {
      cat("Warning: the uploaded data have ", length(idxDuplicatePos),
          " probes with duplicate positions!\n  ", sep = "")
      cat("The BioHMM (with 'use clone distance' selected) and FASeg algorithms have problems analyzing such data.\n")
    }

    if(any(totalProbesPerChrom > maxProbesChrom))  {
      cat("Warning: the uploaded data have at least one chromosome with more than ", maxProbesChrom, " probes.  ", sep = "")
      cat("The CGHseg and cghFLasso algorithms consume large chunks of memory.  If selected, the server will subdivide the data to let those algorithms analyze subsets of the large chromosomes and then reassemble the results.\n")
    }

    if(continueAnalysis)  {
       runAnalysis(tempDir,resultDir,apList, Threshold,        minLR, maxLR, genomeType, tab, methodsIgnored, methodsDivideChrom)
      
      cat("\n")
      #return TRUE
    }else  {
      stop("Since all the methods you specified have been filtered out, no further analysis will be done on the data.\n")

    }



  }else  {
    ## Error message saying that the uploaded file is incorrectly formatted


    cat("----------------------------------------\n")



    stop("Uploaded file in wrong format!\n")
  }
 }

