`drawChromHMGL` <-
function(tab, Z, resType = NULL, zlim = NULL, threshold = NULL,
                          cb = NULL, sampleNames = colnames(Z),
                          dpi = 100)  {
  chr <- tab$Chromosome[1]
  posV <- tab$Position
  drawGL <- ifelse(is.null(threshold), FALSE, TRUE)
  
  ## Set the margins (in inches) of each sub-plot
  mai.ci <-  c(0.50, 0.75, 0.00, 0.10)
  mai.mk <-  c(0.00, 0.75, 0.00, 0.10)
  mai.cb <-  c(0.00, 0.75, 0.00, 0.10)
  mai.vcb <- c(0.10, 0.10, 0.20, 0.45)
  mai.gl <-  c(0.10, 0.10, 0.10, 0.20)

  ## Set the location of each sub-plot by non-dimensional cooridinates (NDC)
  frac <- 14/15
  fig.ci <- c(0, frac, 0, 0.8)
  fig.mk <- c(0, frac, 0.80, 0.85)
  fig.cb <- c(0, frac, 0.85, 1)
  fig.vcb <- c(frac, 1, 0.4, 0.8)
  fig.gl <- c(frac, 1, 0.4, 0.6)
  
  ## Draw either the log-ratio heatmap or the region of gains/losses
  par(mai = mai.ci, fig = fig.ci)

  tmp <- par()
  npix <- round((tmp$fin[1] - tmp$mai[2] - tmp$mai[4])*dpi)
  usePix <- (npix < nrow(Z)) && !drawGL

  if(drawGL)  {
    glImage(Z, resType, threshold, 1, TRUE)
  }else  {
    colV <- bluered(511)
    cghImage(Z, resType, zlim, colV, 1, TRUE, dpi)
  }

  mtext(sampleNames, side = 2, line = 0.25, las = 2, adj = 1, at = 1:ncol(Z), cex = 0.7)
  
  ## Draw position labels at equally spaced indices
  idx.poslabels <- round(seq(1, nrow(Z), length.out = 10))
  posLabels <- formatC(posV[idx.poslabels]/1e6, format = "f", digits = 1)
  if(usePix)  { 
    slopepix <- (npix-1)/(nrow(Z)-1)
    mtext(posLabels, side = 1, line = 0.25, cex = 0.7, at = slopepix*(idx.poslabels-1)+1)
    mtext("Position (Mb)", side = 1, line = 1.5, at = npix/2, cex = 0.9, font = 2)
  }else  {
    mtext(posLabels, side = 1, line = 0.25, cex = 0.7, at = idx.poslabels)
    mtext("Position (Mb)", side = 1, line = 1.5, at = nrow(Z)/2, cex = 0.9, font = 2)
  }
  
  ## Save a list of graphical parameters (to be returned by the function at the end)
  opar <- par()

  ## Draw lines to show the relationship between the probes depicted in the heatmap and genomic position of the cytoband
  par(mai = mai.mk, fig = fig.mk, new = TRUE)
  plot(NA, xlim = range(posV), ylim = c(0,1), xaxs = "i", yaxs = "i", main = "", xlab = "", ylab = "", axes = FALSE)
  x0 <- seq(posV[1], posV[length(posV)], length.out = length(posV))
  idx.subset <- seq(1, length(x0), length.out = round(npix/10))
  segments(x0 = x0[idx.subset], x1 = posV[idx.subset], y0 = 0, y1 = 1, col = "black")

  ## Draw the cytoband
  par(mai = mai.cb, fig = fig.cb, new = TRUE)
  drawCytoBand(cb, chr, range(posV))

  ## Draw either the color bar or the gains/losses legend
  if(drawGL)  {
    par(mai = mai.gl, fig = fig.gl, new = TRUE)
    gllegend()
  }else  {
    par(mai = mai.vcb, fig = fig.vcb, new = TRUE)
    vcbar(zlim, colV)
  }
  
  return(invisible(opar))
}

