`drawChromProfile` <-
function(tab, zlim, cb, equalspacing = FALSE, HMGLmargins = FALSE, dpi = 100)  {
  y <- tab$LogRatio
  posV <- tab$Position
  chr <- tab$Chromosome[1]

  colV <- greenred(511)
  n <- length(colV)
  slope <- (n-1)/(zlim[2]-zlim[1])
  intercept <- 1 - slope*zlim[1]

  idx <- round(slope*y+intercept)
  idx[idx > n] <- n
  idx[idx < 1] <- 1

  if(HMGLmargins)  {
    frac <- 14/15
    mai.profile <-   c(0.50, 0.75, 0.00, 0.10)
    mai.marker <-    c(0.00, 0.75, 0.00, 0.10)
    mai.cytoband <-  c(0.00, 0.75, 0.00, 0.10)
    mai.vcbar <-     c(0.10, 0.10, 0.20, 0.45)
    fig.vcbar <- c(frac, 1, 0.4, 0.8)
  }else  {
    frac <- 1
    mai.profile <-   c(0.50, 0.50, 0.00, 0.25)
    mai.marker <-    c(0.00, 0.50, 0.00, 0.25)
    mai.cytoband <-  c(0.00, 0.50, 0.00, 0.25)
  }
  
  if(equalspacing)  {
    fig.profile <- c(0, frac, 0, 0.80)
    fig.marker <- c(0, frac, 0.80, 0.85)
  }else  {
    fig.profile <- c(0, frac, 0, 0.85)
  }
  fig.cytoband <- c(0, frac, 0.85, 1)

  if(equalspacing)  {
    par(mai = mai.profile, mgp = c(2,0.75,0), fig = fig.profile)
    xlim <- c(0.5, length(y)+0.5)
    plot(y, type = "h", col = colV[idx], xlim = xlim, ylim = zlim,
         xaxs = "i", yaxs = "i", xlab = "", ylab = "",
         main = "", axes = FALSE)
    axis(2, cex.axis = 0.7)
    idx.poslabels <- round(seq(1, length(y), length.out = 10))
    mtext(formatC(posV[idx.poslabels]/1e6, format = "f", digits = 1), side = 1, line = 0.25, cex = 0.7, at = idx.poslabels)
    mtext("Position (Mb)", side = 1, line = 1.5, at = mean(xlim), cex = 0.9, font = 2)
    mtext("Log-Ratio", side = 2, line = 2, at = mean(zlim), cex = 0.9, font = 2)
    box()

    opar <- par()

    ## Draw lines that denote the placement of probes to their respective
    ## genomic positions on the cytogenetic band
    par(mai = mai.marker, fig = fig.marker, new = TRUE)
    plot(NA, xlim = range(posV), ylim = c(0,1), xaxs = "i", yaxs = "i", main = "", xlab = "", ylab = "", axes = FALSE)
    x0 <- seq(posV[1], posV[length(posV)], length.out = length(posV))
    tmp <- par()
    npix <- round((tmp$fin[1] - tmp$mai[2] - tmp$mai[4])*dpi)
    idx.subset <- seq(1, length(x0), length.out = round(npix/10))
    segments(x0 = x0[idx.subset], x1 = posV[idx.subset], y0 = 0, y1 = 1, col = "black")
  }else  {
    par(mai = mai.profile, mgp = c(2,0.75,0), fig = fig.profile)
    xlim <- range(posV)
    plot(x = posV, y = y, type = "h", col = colV[idx], xlim = xlim, ylim = zlim,
         xaxs = "i", yaxs = "i", xlab = "", ylab = "",
         main = "", axes = FALSE)
    axis(2, cex.axis = 0.7)
    posLabels <- seq(xlim[1], xlim[2], length.out = 10)
    mtext(formatC(posLabels/1e6, format = "f", digits = 1), side = 1, line = 0.25, cex = 0.7, at = posLabels)
    mtext("Position (Mb)", side = 1, line = 1.5, at = mean(xlim), cex = 0.9, font = 2)
    mtext("Log-Ratio", side = 2, line = 2, at = mean(zlim), cex = 0.9, font = 2)
    box()

    opar <- par()
  }

  par(mai = mai.cytoband, fig = fig.cytoband, new = TRUE)
  drawCytoBand(cb, chr, range(posV))

  ## If the user chooses to use the same margins used in making the heatmaps,
  ## draw a vertical color bar within the right margin.
  if(HMGLmargins)  {
    par(mai = mai.vcbar, fig = fig.vcbar, new = TRUE)
    vcbar(zlim, colV)
  }
  
  return(invisible(opar))
}

