`drawCytoBand` <-
function(cb, chr, xlim)  {
  cbchr <- cb[cb[[1]] == paste("chr", chr, sep = ""),]
  sp <- cbchr[[2]] + 1
  ep <- cbchr[[3]]

  ## Find the cytoband regions that are within xlim, inclusively
  idx.LB <- max(c(1, which(sp <= xlim[1])))
  idx.UB <- min(c(which(ep >= xlim[2]), length(ep)))
  cbchr <- cbchr[idx.LB:idx.UB, ]
  sp <- cbchr[[2]] + 1
  ep <- cbchr[[3]]
  
  colchr <- rep(NA, nrow(cbchr))
  typechr <- rep(NA, nrow(cbchr))

  ## The color schemes for gvar, stalk, and acen are somewhat arbitrary.
  colchr[cbchr[[5]] == "gneg"] <- "white"
  colchr[cbchr[[5]] == "gpos"] <- "black"
  colchr[cbchr[[5]] == "acen"] <- "brown"
  colchr[cbchr[[5]] == "gvar"] <- gray(0.875)
  colchr[cbchr[[5]] == "stalk"] <- gray(0.875)

  idx.gposNum <- grep("gpos[0-9]*", cbchr[[5]])
  colchr[idx.gposNum] <-
    gray(1 - as.numeric(sub("gpos", "", cbchr[[5]][idx.gposNum]))/100)

  typechr[grep("^g", cbchr[[5]])] <- "rect"
  typechr[cbchr[[5]] == "acen"] <- "acen"
  typechr[cbchr[[5]] == "stalk"] <- "stalk"

  ## The actual cytoband only takes up 1 unit in the y-axis, but we
  ## reserve more space so that we can label the cytoband.
  plot(NA, xlim = xlim, ylim = c(0,3), xaxs = "i", yaxs = "i", main = "",
       xlab = "", ylab = "", axes = FALSE)

  ## The below lines tells the code what type of triangle to draw first
  ## for "acen":
  ##
  ## (a) If the boundaries contain two acentric regions, draw two triangles,
  ##     the former pointing to the right and the latter point to the left.
  ## (b) If the boundaries contain one acentric region, and this region is
  ##     at the right-most boundary, draw one triangle pointing to the right.
  ## (c) Otherwise, draw one triangle pointing to the left.
  if(sum(typechr == "acen") == 2)
    acen <- FALSE
  else if(typechr[length(typechr)] == "acen")
    acen <- FALSE
  else
    acen <- TRUE

  for(i in 1:nrow(cbchr))  {
    if(typechr[i] == "rect")  {
      rect(xleft = sp[i], xright = ep[i],
           ybottom = 0, ytop = 1, col = colchr[i])
    }else if(typechr[i] == "stalk")  {
      ## Draw a "bow-tie"
      dp <- ep[i] - sp[i] + 1
      polygon(x = sp[i] + dp*c(0,0.25,0.75,1,1,0.75,0.25,0),
              y = c(1,0.75,0.75,1,0,0.25,0.25,0), col = colchr[i])
    }else  {
      if(acen)  {
        ## Left-pointing triangle
        polygon(x = c(sp[i], ep[i], ep[i]), y = c(0.5,1,0), col = colchr[i])
      }else  {
        ## Right-pointing triangle
        polygon(x = c(sp[i], ep[i], sp[i]), y = c(1,0.5,0), col = colchr[i])
        acen <- TRUE
      }
    }
  }

  ## Label the cytobands
  tmpsize <- ep-sp+1
  mp <- (sp+ep)/2
  txt <- cbchr[[4]]
  text(x = mp, y = 1.25, cex = 0.5, labels = txt, col = "blue",
       offset = 0, pos = 4, srt = 45, xpd = NA)
}

