`drawGenomeHMGL` <-
function(tab, Z, resType = NULL, zlim = NULL, threshold = NULL,
                           cb = NULL, sampleNames = colnames(Z),
                           dpi = 100)  {
  chrV <- tab$Chromosome
  unique.chrs <- unique(chrV)
  drawGL <- ifelse(is.null(threshold), FALSE, TRUE)

  ep <- which(diff(as.integer(factor(chrV))) != 0)
  sp <- c(1, ep+1)
  ep <- c(ep, nrow(tab))
  mp <- (sp+ep)/2
  
  ## Set the margins (in inches) of each sub-plot
  mai.ci <-  c(0.50, 0.75, 0.25, 0.10)
  mai.vcb <- c(0.10, 0.10, 0.20, 0.45)
  mai.gl <-  c(0.10, 0.10, 0.10, 0.20)

  ## Set the location of each sub-plot by non-dimensional cooridinates (NDC)
  frac <- 14/15
  fig.ci <- c(0, frac, 0, 1)
  fig.vcb <- c(frac, 1, 0.4, 0.8)
  fig.gl <- c(frac, 1, 0.4, 0.6)
  
  ## Draw either the log-ratio heatmap or the region of gains/losses
  par(mai = mai.ci, fig = fig.ci)

  tmp <- par()
  npix <- round((tmp$fin[1] - tmp$mai[2] - tmp$mai[4])*dpi)
  usePix <- (npix < nrow(Z)) && !drawGL

  if(drawGL)  {
    glImage(Z, resType, threshold, 1, TRUE)
  }else  {
    colV <- bluered(511)
    cghImage(Z, resType, zlim, colV, 1, TRUE, dpi)
  }

  if(usePix)  {
    slopepix <- (npix-1)/(nrow(Z)-1)
    abline(v = slopepix*((sp[-1]-0.5) - 1) + 1)
    mtext(unique.chrs, side = rep(c(1,3), length.out = length(unique.chrs)), at = slopepix*(mp - 1) + 1, line = 0.25, cex = 0.7)
    mtext("Chromosome", side = 1, line = 1.5, at = npix/2, cex = 0.9, font = 2)
  }else  {
    abline(v = sp[-1] - 0.5)
    mtext(unique.chrs, side = rep(c(1,3), length.out = length(unique.chrs)), at = mp, line = 0.25, cex = 0.7)
    mtext("Chromosome", side = 1, line = 1.5, at = nrow(tab)/2, cex = 0.9, font = 2)
  }
  mtext(sampleNames, side = 2, line = 0.25, las = 2, adj = 1, at = 1:ncol(Z), cex = 0.7)
  
  ## Save a list of graphical parameters (to be returned by the function at the end)
  opar <- par()

  ## Draw either the color bar or the gains/losses legend
  if(drawGL)  {
    par(mai = mai.gl, fig = fig.gl, new = TRUE)
    gllegend()
  }else  {
    par(mai = mai.vcb, fig = fig.vcb, new = TRUE)
    vcbar(zlim, colV)
  }
  
  return(invisible(opar))
}

