`drawGenomeProfile` <-
function(tab, zlim, HMGLmargins = FALSE)  {
  y <- tab$LogRatio
  chrV <- tab$Chromosome
  unique.chrs <- unique(chrV)
  
  ep <- which(diff(as.integer(factor(chrV))) != 0)
  sp <- c(1, ep+1)
  ep <- c(ep, nrow(tab))
  mp <- (sp+ep)/2

  colV <- greenred(511)
  n <- length(colV)
  slope <- (n-1)/(zlim[2]-zlim[1])
  intercept <- 1 - slope*zlim[1]

  idx <- round(slope*y+intercept)
  idx[idx > n] <- n
  idx[idx < 1] <- 1

  if(HMGLmargins)  {
    mai.profile <-   c(0.50, 0.75, 0.25, 0.10)
    fig.profile <- c(0, 14/15, 0, 1)
    mai.vcbar <-     c(0.10, 0.10, 0.20, 0.45)
    fig.vcbar <- c(14/15, 1, 0.4, 0.8)
  }else  {
    mai.profile <-   c(0.50, 0.50, 0.25, 0.25)
    fig.profile <- c(0, 1, 0, 1)
  }
  
  par(mai = mai.profile, mgp = c(2,0.75,0), fig = fig.profile)
  plot(y, type = "h", col = colV[idx], xlim = c(0.5, nrow(tab)+0.5), ylim = zlim,
       xaxs = "i", yaxs = "i", xlab = "", ylab = "",
       main = "", axes = FALSE)
  abline(v = sp[-1] - 0.5, col = "blue", lty = 2)
  axis(2, cex.axis = 0.7)

  mtext(unique.chrs, side = rep(c(1,3), length.out = length(unique.chrs)), at = mp, line = 0.25, cex = 0.7)
  mtext("Chromosome", side = 1, line = 1.5, at = nrow(tab)/2, cex = 0.9, font = 2)
  mtext("Log-Ratio", side = 2, line = 2, at = mean(zlim), cex = 0.9, font = 2)
  box()

  opar <- par()
  
  if(HMGLmargins)  {
    par(mai = mai.vcbar, fig = fig.vcbar, new = TRUE)
    vcbar(zlim, colV)
  }

  return(invisible(opar))
}

