`findGNL` <-
function(y, chrV = rep(1, length(y)), threshold)  {
  idx.nona <- which(!is.na(y))
  y.nona <- y[idx.nona]
  chrV.nona <- chrV[idx.nona]

  rV <- numeric(length(y.nona))
  rV[y.nona >=  threshold] <-  1
  rV[y.nona <= -threshold] <- -1

  chrs <- unique(chrV.nona)
  sp.g <- ep.g <- numeric(0)
  for(chr in chrs)  {
    idx.chr <- which(chrV.nona == chr)
    rV.chr <- rV[idx.chr]
    ep <- which(diff(rV.chr) != 0)

    if(length(ep) > 0)  {
      ## Case: at least 2 copy number types
      sp <- c(1, ep+1)
      ep <- c(ep, length(idx.chr))
      sp.g <- c(sp.g, sp + idx.chr[1] - 1)
      ep.g <- c(ep.g, ep + idx.chr[1] - 1)
    }else  {
      ## Case: only 1 copy number type
      sp.g <- c(sp.g, idx.chr[1])
      ep.g <- c(ep.g, idx.chr[length(idx.chr)])
    }
  }

  meanSegments <- numeric(length(sp.g))
  for(i in 1:length(sp.g))
    meanSegments[i] <- mean(y.nona[sp.g[i]:ep.g[i]])

  gnlTable <- data.frame(Chr = chrV.nona[sp.g], StartIdx = idx.nona[sp.g], EndIdx = idx.nona[ep.g], Mean = meanSegments)
  loss <-   gnlTable[rV[sp.g] == -1, , drop = FALSE]
  normal <- gnlTable[rV[sp.g] ==  0, , drop = FALSE]
  gain <-   gnlTable[rV[sp.g] ==  1, , drop = FALSE]

  return(list(loss = loss, normal = normal, gain = gain))
}

