`makeResultsHTML` <-
function(tab,tempDir=getwd(),resultDir="CGHResults", genomeType,nredraw,resultsDir,resultsFN,
          NewminLR,NewmaxLR,NewThreshold,Z,resType,methodNames,methodsIgnored,
              tabFNgzip, startTime, endTime, fsize)
{
    tDir=file.path(tempDir,resultDir)
    cbFN <- toupper(genomeType)
    data(cytoband)
       if (!(cbFN=="HG18"||cbFN=="HG17"||cbFN=="HG16"||cbFN=="HG15"||cbFN=="MM5"||cbFN=="MM6"||cbFN=="MM7"||cbFN=="MM8"||cbFN=="RN3"||cbFN=="RN4"))
     {  stop ("Invalid genome type")}
     
     if (cbFN=="HG18")
     { cb<-HG18}
     if (cbFN=="HG17")
     { cb<-HG17}
     if (cbFN=="HG16")
     { cb<-HG16}
     if (cbFN=="HG15")
     { cb<-HG15}
     if (cbFN=="MM5")
     { cb<-MM5}
     if (cbFN=="MM6")
     { cb<-MM6}
     if (cbFN=="MM7")
     { cb<-MM7}
     if (cbFN=="MM8")
     { cb<-MM8}
     if (cbFN=="RN3")
     { cb<-RN3}
     if (cbFN=="RN4")
     { cb<-RN4}

        cat("Making HTML pages\n")
  
    
    ## Load in GD graphical package and relevant R scripts
    #source(file.path(rScriptsDir, "cghImage.R"))
    #source(file.path(rScriptsDir, "calcCoords.R"))
   # source(file.path(rScriptsDir, "FormatDF.R"))
   ## Load cytoband annotations
    #data(cb)
    #cb <- read.delim(file.path(cbDir, cbFN), as.is = TRUE, header = FALSE)
    
    cat("Loaded cytoband annotation","\n",sep="");
    
    ## Find start and stop indices of each chromosome
    tmp <- as.integer(factor(tab$Chrom))
    eidx <- which(diff(tmp) != 0)
    sidx <- c(1, eidx + 1)
    eidx <- c(eidx, nrow(tab))
    #cat("Found start and stop indices of each chromosome","\n",sep="")

    #load(file.path(tDir,"ExtraParams.RData"))
    ## Create graphical log-ratio profile of original data for genome
    zlim <- c(NewminLR, NewmaxLR)
    profileFN.genome.original <- "profile_genome_original.png"
    GDD(file.path(resultsDir, profileFN.genome.original), width = 12*100, height = 2*100)
    oparGenome <- drawGenomeProfile(tab, zlim, TRUE)
    dev.off()
    #cat("Graphical log ratio created for original data","\n",sep="")
    
    ## Create graphical log-ratio profile of summary data for genome
    
    #load(file.path(tDir,"redraw.RData"))
    
    profileFN.genome.summary <- "profile_genome_summary.png"
    GDD(file.path(resultsDir, profileFN.genome.summary), width = 12*100, height = 2*100)
    tabSummary <- tab
    tabSummary$LogRatio <- Z[,"Summary"]
    drawGenomeProfile(tabSummary, zlim, TRUE)
    dev.off()
    cat("Graphical log ratio of summary data","\n",sep="")
    #flush.console()
    
    ## Draw heatmap of the genome
    hmapFN.genome <- "heatmap_genome.png"
    GDD(file.path(resultsDir, hmapFN.genome), width = 12*100, height = 4*100)
    oparHM <- drawGenomeHMGL(tab, Z, resType, zlim, NULL, cb)
    dev.off()
    cat("Heat map drawn","\n",sep="")
    
    
    ## Draw a genomic map of gains and losses
    glFN.genome <- "gainloss_genome.png"
    GDD(file.path(resultsDir, glFN.genome), width = 12*100, height = 4*100)
    drawGenomeHMGL(tab, Z, resType, zlim, NewThreshold, cb)
    dev.off()
    #cat("genomic maps of gains and loss drawn","\n",sep="")
    
    
    ## Calculate coordinates for genome profile
    idx <- c(sidx, nrow(tab))
    coords <- calcCoords(idx, zlim, res = 100, opar = oparGenome)
    coordsGenomeP <- paste(coords$x[-length(idx)], coords$y[2], coords$x[-1], coords$y[1], sep = ",")
    #cat("Corodrinates for genomic profile calc","\n",sep="")
    
    
    ## Calculate coordinates for heatmap of the genome
    npix <- round((oparHM$fin[1] - oparHM$mai[2] - oparHM$mai[4])*100)
    slopepix <- (npix-1)/(nrow(Z)-1)
    if(nrow(Z) > npix)  {
      coords <- calcCoords(slopepix*(idx-1)+1, c(ncol(Z)+0.5, 0.5), res = 100, opar = oparHM)
    }else  {
      coords <- calcCoords(idx, c(ncol(Z)+0.5, 0.5), res = 100, opar = oparHM)
    }
    coordsGenomeHM <- paste(coords$x[-length(idx)], coords$y[2], coords$x[-1], coords$y[1], sep = ",")
    
    ## (a) Create graphical log-ratio profile of original data for each chromosome
    ## (b) Create graphical log-ratio profile of summary data for each chromosome
    ## (c) Draw heatmap for each chromosome
    ## (d) Draw map of gains and losses for each chromosome
    unique.chrs <- unique(tab$Chrom)
    profileFN.chrV.original <- paste("profile_chr", unique.chrs, "_original.png", sep = "")
    profileFN.chrV.summary <- paste("profile_chr", unique.chrs, "_summary.png", sep = "")
    hmapFN.chrV <- paste("heatmap_chr", unique.chrs, ".png", sep = "")
    glFN.chrV <- paste("gainloss_chr", unique.chrs, ".png", sep = "")
    
    for(i in 1:length(unique.chrs))  {
      idx.chr <- sidx[i]:eidx[i]
      tab.chr <- tab[idx.chr, ]
      Z.chr <- Z[idx.chr, ]
      tabSummary.chr <- tabSummary[idx.chr, ]
      cat("Chromosome ", unique.chrs[i], "\n", sep = "")
      #flush.console()
      GDD(file.path(resultsDir, profileFN.chrV.original[i]), width = 12*100, height = 4*100)
      oparChrom <- drawChromProfile(tab.chr, zlim, cb, FALSE, TRUE)
      dev.off()
    
      GDD(file.path(resultsDir, profileFN.chrV.summary[i]), width = 12*100, height = 4*100)
      drawChromProfile(tabSummary.chr, zlim, cb, FALSE, TRUE)
      dev.off()
    
      GDD(file.path(resultsDir, hmapFN.chrV[i]), width = 12*100, height = 4*100)
      oparChromHM <- drawChromHMGL(tab.chr, Z.chr, resType, zlim, NULL, cb)
      dev.off()
    
      GDD(file.path(resultsDir, glFN.chrV[i]), width = 12*100, height = 4*100)
      drawChromHMGL(tab.chr, Z.chr, resType, zlim, NewThreshold, cb)
      dev.off()
    }
    
    ## Calculate coordinates for the chromosome plots
    npix <- round((oparChrom$fin[1] - oparChrom$mai[2] - oparChrom$mai[4])*100)
    numBlocks <- round(npix/10)
    tmp <- seq(oparChrom$usr[1], oparChrom$usr[2], length.out = numBlocks+1)
    coords <- calcCoords(tmp, oparChrom$usr[3:4], res = 100, opar = oparChrom)
    coordsChromP <- paste(coords$x[-length(tmp)], coords$y[2], coords$x[-1], coords$y[1], sep = ",")
    
    tmp <- seq(oparChromHM$usr[1], oparChromHM$usr[2], length.out = numBlocks+1)
    coords <- calcCoords(tmp, oparChromHM$usr[3:4], res = 100, opar = oparChromHM)
    coordsChromHM <- paste(coords$x[-length(tmp)], coords$y[2], coords$x[-1], coords$y[1], sep = ",")
    
    ## Close log
    #make the time graph here.
    timedata<-as.matrix(read.delim(file.path(tDir,"algorithmTimes.txt"),header=FALSE,sep="\t"))
    GDD(file.path(tDir,"TimeUsage.png"),	width =  500, height = 300)
    par(las=3,ps=8)
    barplot(as.numeric(timedata[,2]),names.arg = timedata[,1], beside =TRUE,col="orange", ylab="Time in sec",main="Time taken by algorithms") 
    
    dev.off()   
    
    ## Make HTML summaries of graphical results
    ## (a) HTML page of genome
    sink(file.path(resultsDir, resultsFN))
    ParamFN <-paste("ParametersSelected.html")
    TimeFN <-paste("TimeUsage.png")
    
    cat("<html>\n")
    cat("<head><title>Results of aCGH analysis</title><link rel='stylesheet' href='http://compbio.med.harvard.edu/CGHweb/styles/cghweb.css' type='text/css'></head>\n")
    cat("<body >\n")
    cat("<A name=\"top\"></A>\n")
    tgzDir=tDir
    cat ("<table width ='100%' border='0'><tr><td>")

    if(!is.null(methodsIgnored))
    cat("<br><b><font  color='red' size =2>Warning: The following methods have been ignored: <font color ='black'><u>", paste(methodNames[methodsIgnored], collapse = ", "), "</u></font></font></b>\n", sep = "")
    cat("<br><a href=\"Regions_of_Gains_and_Losses.html\">Tabular representation of \"Regions of Gains and Losses\" for the algorithms selected</a>\n", sep = "")
    cat("<br><a href=\"",file.path(tgzDir, tabFNgzip), "\">", "Table of Smoothed Profiles (gzip compressed: ", fsize, " kB)", "</a>\n", sep = "")
    cat("<br>Calculations Started: ", startTime, "\n", sep = "")
    cat("<br>Calculations Ended: ", endTime, "\n", sep = "")
    cat ("</td><td valign='top' height='187'> <div align='right'><img src=\"",TimeFN,"\" width='500' height='300'></div>  </td> </tr></table>",sep ="")
    
    cat("<font size =5><center><b><u>Genomic overview</u></b><br> Click on chromosomes to see detailed view</center></font><br>\n")
    cat("<b><u><font size =3>Original Data:</font></u></b><br>Chromosome numbers are given on top and bottom of the image. Log-ratios are indicated by both the y-axis and the color (green indicates regions of chromosomal loss and red indicates regions of chromosomal gain).Click on the figure for a detailed view of a particular chromosome.<br>\n")
    cat("<img src=\"", profileFN.genome.original, "\" usemap=\"#profile\" border=0>\n", sep = "")
    cat("<br><b><u><font size =3>Summary Data(Pointwise averaging of all computed profiles):</font></u></b><br><br>\n")
    cat("<img src=\"", profileFN.genome.summary, "\" usemap=\"#profile\" border=0>\n", sep = "")
    
    cat("<MAP name=\"profile\">\n")
    for(i in 1:length(unique.chrs))  {
      chrFN <- paste("results_chr", unique.chrs[i], ".html", sep = "")
      cat("<AREA href=\"", chrFN, "\" shape=\"rect\" coords=\"", coordsGenomeP[i], "\">\n", sep = "")
    }
    cat("</MAP>\n")
    
    cat("<hr><b><u><font size =3>Heatmap of Log-Ratios for Original, Smoothed/Segmented, and Summary Data:</font></u></b><br>The first line is the heatmap of the original log-ratios; the last is the heatmap of the averaged profile (pointwise averaging across the outputs of all algorithms); and the lines in the middle are the heatmaps corresponding to different algorithms.<br>\n")
    cat("<img src=\"", hmapFN.genome, "\" usemap=\"#hmap\" border=0>\n", sep = "")
    cat("<hr><b><u><font size =3>Map of Gains/Losses for Smoothed/Segmented and Summary Data</font></u></b><br>The log-ratios in the heatmap above are further classified as gain or loss, based on the threshold specified by the user. This plot can be redrawn with a different threshold by entering the new parameters at the bottom of the page and clicking \"Redraw plots.\"<br>\n")
    cat("<img src=\"", glFN.genome, "\" usemap=\"#hmap\" border=0>\n", sep = "")
    
    cat("<MAP name=\"hmap\">\n")
    for(i in 1:length(unique.chrs))  {
      chrFN <- paste("results_chr", unique.chrs[i], ".html", sep = "")
      cat("<AREA href=\"", chrFN, "\" shape=\"rect\" coords=\"", coordsGenomeHM[i], "\">\n", sep = "")
    }
    cat("</MAP>\n")
  
    
    cat("</body>\n")
    cat("</html>\n")
    sink()
    
    ## (b) HTML page of each chromosome
    ##
    ## Note: the link to the UCSC genome browser is
    ##       http://genome.ucsc.edu/cgi-bin/hgTracks?org=[org]&db=[db]&position=[position]
    ##
    ##       The "org" portion is optional if "db" is supplied
    ##
    ## The below code is similar to the HTML output code used in zoomChr.R
    UCSC <- paste("http://genome.ucsc.edu/cgi-bin/hgTracks?db=", tolower(genomeType), "&position=chr", sep = "")
    
    for(i in 1:length(unique.chrs))  {
      chr <- unique.chrs[i]
      sink(file.path(resultsDir, paste("results_chr", chr, ".html", sep = "")))
      cat("<html>\n")
      cat("<head><title>CGHweb:Chromosome ", chr, "</title><link rel='stylesheet' href='http://compbio.med.harvard.edu/CGHweb/styles/cghweb.css' type='text/css'></head>\n", sep = "")
      cat("<body >\n")
      cat("<Font  size =5><b><center>Analysis of chromosome number ", chr, "</b><br>Click on image to see details on UCSC genome browser</center></font>\n", sep = "")
      cat("<b><u><font size =3>Original Data:</font></u></b>\n")
      cat("&nbsp;&nbsp;&nbsp;&nbsp;<font face = 'verdana' size =1><A href=\"", resultsFN, "\">Back to genome</A></font><br>Cytogenetic Ideogram for human chromosome is shown on the top. Position in Mb along the chromosome arm is shown at the bottom and log ratio is on the left. Green indicates regions of frequent chromosomal loss and red indicates regions of frequent chromosomal gain. Clicking on the map will connect to UCSC genome browser for a specific region<br>\n", sep = "")
      cat("<img src=\"", profileFN.chrV.original[i], "\" usemap=\"#profile\" border=0>\n", sep = "")
      cat("<b><u><font size =3>Summary Data(Pointwise averaging of all computed profiles):</font></u></b>\n")
      cat("&nbsp;&nbsp;&nbsp;&nbsp;<font face = 'verdana' size =1><A href=\"", resultsFN, "\">Back to genome</A></font><br><br>\n", sep = "")
      cat("<img src=\"", profileFN.chrV.summary[i], "\" usemap=\"#profile\" border=0>\n", sep = "")
    
      cat("<MAP name=\"profile\">\n")
      tmppos <- round(seq(tab$Position[sidx[i]], tab$Position[eidx[i]], length.out = numBlocks+1))
      ucscLink <- paste(UCSC, chr, ":", tmppos[-(numBlocks+1)], "-", tmppos[-1], sep = "")
      tmp <- paste("<AREA href=\"", ucscLink, "\" shape=\"rect\" coords=\"", coordsChromP, "\" target=\"_BLANK\">\n", sep = "")
      cat(tmp, sep = "")
      cat("</MAP>\n")
      
      cat("<hr><b><u><font size =3>Heatmap of Log-Ratios for Original, Smoothed/Segmented, and Summary Data:</font></u></b>\n")
        cat("&nbsp;&nbsp;&nbsp;&nbsp;<font face = 'verdana' size =1><A href=\"", resultsFN, "\">Back to genome</A></font><br>Cytogenetic Ideogram for human chromosome is shown on the top. Position in Mb along the chromosome arm is shown at the bottom and algorithms used on the left. Red indicates gains and blue indicates loss. Clicking on the map will connect to UCSC genome browser for a specified region <br>\n", sep = "")
      cat("<img src=\"", hmapFN.chrV[i], "\" usemap=\"#hmap\" border=0>\n", sep = "")
      cat("<hr><b><u><font size =3>Map of Gains/Losses for Smoothed/Segmented and Summary Data:</font></u></b>\n")
        cat("&nbsp;&nbsp;&nbsp;&nbsp;<font face = 'verdana' size =1><A href=\"", resultsFN, "\">Back to genome</A></font><br>Cytogenetic Ideogram for human chromosome is shown on the top. Position in Mb along the chromosome arm is shown at the bottom and algorithms used on the left. Orange indicates gains and green indicates loss(based on user specific threshold). Clicking on the map will connect to UCSC genome browser for a specified region <br>\n", sep = "")
      cat("<img src=\"", glFN.chrV[i], "\" usemap=\"#hmap\" border=0>\n", sep = "")
    
      cat("<MAP name=\"hmap\">\n")
      tmppos <- tab$Position[round(seq(sidx[i], eidx[i], length.out = numBlocks+1))]
      ucscLink <- paste(UCSC, chr, ":", tmppos[-(numBlocks+1)], "-", tmppos[-1], sep = "")
      tmp <- paste("<AREA href=\"", ucscLink, "\" shape=\"rect\" coords=\"", coordsChromHM, "\" target=\"_BLANK\">\n", sep = "")
      cat(tmp, sep = "")
      cat("</MAP>\n")
    
   
      cat("</body>\n")
      cat("</html>\n")
      sink()
    }
    
    
    ## Summarize regions of gains and losses in tabular format
    annotateGNLTable <- function(gnlTable, LogRatio, ProbeID, Position)  {
      pid.start <- ProbeID[gnlTable$StartIdx]
      pid.end <- ProbeID[gnlTable$EndIdx]
    
      pos.start <- Position[gnlTable$StartIdx]
      pos.end <- Position[gnlTable$EndIdx]
    
      numProbes <- gnlTable$EndIdx - gnlTable$StartIdx + 1
    
      numValidProbes <- numeric(nrow(gnlTable))
      nonaV <- as.numeric(!is.na(LogRatio))
      for(i in 1:nrow(gnlTable))
        numValidProbes[i] <- sum(nonaV[gnlTable$StartIdx[i]:gnlTable$EndIdx[i]])
    
      atab <- data.frame(Chrom = gnlTable$Chr,
                         StartProbeID = pid.start, EndProbeID = pid.end,
                         StartIndex = gnlTable$StartIdx, EndIndex = gnlTable$EndIdx,
                         StartPosition = pos.start, EndPosition = pos.end,
                         NumProbes = numProbes, NumValidProbes = numValidProbes,
                         Mean = gnlTable$Mean)
      return(atab)
    }
    
    
    sink(file.path(resultsDir, "Regions_of_Gains_and_Losses.html"))
    cat("<HTML>\n")
    cat("<HEAD><TITLE>CGHweb: Regions of Gains and Losses</TITLE><link rel='stylesheet' href='http://compbio.med.harvard.edu/CGHweb/styles/cghweb.css' type='text/css'></HEAD>\n")
    cat("<BODY >\n")
    cat("<A name=\"top\"></A>\n")
    cat("<center><b><u>Regions of Gains and Losses</u></b></center>\n")
    for(i in 1:ncol(Z))  {
      if(resType[i] == "SLR")
        cat("<b><a href=\"#", colnames(Z)[i], "\">", colnames(Z)[i], "</b></a>&nbsp;&nbsp;&nbsp;\n", sep = "")
        
    }
    cat("<hr>\n")
    
    digits <- c(0,0,0,0,0,0,0,0,0,3)
    
    for(i in 1:ncol(Z))  {
      if(resType[i] == "SLR")  {
        gnlList <- findGNL(Z[,i], tab$Chrom, NewThreshold)
        cat("<a name=\"", colnames(Z)[i], "\">", colnames(Z)[i], "</a>\n", sep = "")
    
        cat("<br><b><u>Regions of Gains</u></b><br>\n")
        if(nrow(gnlList$gain) > 0)  {
          tmp <- annotateGNLTable(gnlList$gain, tab$LogRatio, tab$ProbeID, tab$Position)
          cat(FormatDF(tmp, digits, FALSE), sep = "\n")
        }else
          cat("<p>None identified</p>\n")
        cat("\n")
    
        cat("<br><b><u>Regions of Losses</u></b><br>\n")
        if(nrow(gnlList$loss) > 0)  {
          tmp <- annotateGNLTable(gnlList$loss, tab$LogRatio, tab$ProbeID, tab$Position)
          cat(FormatDF(tmp, digits, FALSE), sep = "\n")
        }else
          cat("<p>None identified</p>\n")
        cat("\n")
    
        cat("<font face ='verdana' size =1><A href=\"#top\">Back to top of page</A></font>\n")
        cat("<hr>\n")
      }
    }
    
    cat("</BODY>\n")
    cat("</HTML>\n")
    sink()
    cat ("-------------------------------------------------\n")
    cat ("Analysis Done:\nPlease open ",file.path(tDir,"results.html"), " in the browers to see analysis\n ",sep="")
     
}

