`runAnalysis` <-
function(tempDir,resultDir,apList, Threshold,   minLR, maxLR, genomeType, tab, methodsIgnored, methodsDivideChrom)
{
    uDir <- resultDir
    tDir <- file.path(tempDir, uDir)
    
    ## Begin log
 
    startTime <- date()
    cat("----------------------------------------\n")
    cat("Starting Analysis\n")
    
    
    ## Read in relevant R scripts
    #source(file.path(rScriptsDir, "algorithms_all.R"))
   # source(file.path(rScriptsDir, "rowMedians.R"))
   # source(file.path(rScriptsDir, "divideChrom.R"))
    
    ## Import saved variables ("R workspaces")
    #load(file.path(tDir, "inputDF.RData"))
      maxProbesChrom<-10000
    nm <- length(apList)
    res <- matrix(NA, nrow = nrow(tab), ncol = nm)
    methodNames <- c("Lowess", "Wavelet", "Quantreg", "Runavg", "CBS", "CGHseg", "BioHMM", "cghFLasso", "GLAD", "FASeg", "Hu", "CGHclassify")
    names(methodNames) <- c("calcLowess", "calcWavelet", "calcQuantreg", "calcRunavg", "calcCBS", "calcPicard", "calcBioHMM", "calcFL", "calcGLAD", "calcFASeg", "calcHu", "calcCGHclassify")
    colnames(res) <- methodNames[names(apList)]
    
    inputDataList <- list(y = tab$LogRatio, chrs = tab$Chrom, spos = tab$Position, epos = tab$Position)
    
    if(!is.null(methodsDivideChrom))  {
      inputDataListAlt <- inputDataList
    
      inputDataListAlt$chrs <- divideChrom(tab$Chrom, maxProbesChrom)
    }
    
    timeSpent <- numeric(nm)
    
    ## resType can denote either
    ##   * SLR:    segmented/smoothed log-ratio
    ##   * OLR:    original log-ratio
    ##   * GNL:    gain,normal,loss indicator vector (e.g., from CGHclassify)
    ##   * error:  no results because of algorithm error (e.g., from BioHMM)
    resType <- character(nm)
    
    ## Report algorithm run-times to separate text file
    zz <- file(file.path(tDir, "algorithmTimes.txt"), "wt")
    
    for(j in 1:nm)  {
      if(names(apList)[j] %in% methodsDivideChrom)  {
        timeSpent[j] <- system.time(tmp <-
                                    do.call(names(apList)[j],
                                            c(inputDataListAlt, apList[[j]])))["elapsed"]
      }else  {
        timeSpent[j] <- system.time(tmp <-
                                    do.call(names(apList)[j],
                                            c(inputDataList, apList[[j]])))["elapsed"]
      }
      resType[j] <- attr(tmp, "dataType")
    
      if(resType[j] != "error")  {
        cat(names(apList)[j], ": ", timeSpent[j], " sec\n", sep = "")
        writeLines(paste(colnames(res)[j], "\t", timeSpent[j], sep = ""), zz)
        
      }else  {
        cat(names(apList)[j], ": algorithm error!\n", sep = "")
        writeLines(paste(colnames(res)[j], "\t", "error", sep = ""), zz)
       
      }
      res[,j] <- tmp
    }
    close(zz)
    
    ## Create a summary column across different
    ## array CGH algorithm outputs by taking their median
    ## res <- cbind(res, Summary = apply(res, 1, median, na.rm = TRUE))
    ##
    ## Note: rowMedians() is a much faster alternative to apply()
    res <- cbind(res, Summary = rowMedians(res))
    resType <- c(resType, "SLR")
    
    ## Create downloadable table in tab-delimited format (compressed by GZIP)
    tabres <- cbind(tab, res)
    tabFN <- "Table_of_aCGH_smoothed_profiles.txt"
    tabFNgzip <- paste(tabFN, ".gz", sep = "")
    
    zz <- gzfile(file.path(tDir, tabFNgzip), "w")
    write.table(tabres, zz, sep = "\t", quote = FALSE, na = "", row.names = FALSE, col.names = TRUE)
    close(zz)                                                      
    
    fsize <- ceiling(file.info(file.path(tDir, tabFNgzip))$size / 1024)
    
    ## Save data needed to redraw figures
   Z <- cbind(Original = tab$LogRatio, res)
    resType <- c("OLR", resType)
    #rdataFN <- "redraw.RData"
   endTime <- date()
  #  save(Z, resType, methodNames, methodsIgnored, tabFNgzip, startTime, endTime, fsize, file = file.path(tDir, rdataFN))
    
    ## Save data needed to quickly zoom in on the figures
    #unique.chrs <- unique(tab$Chrom)
    #for(chr in unique.chrs)  {
    #  idx.chr <- tab$Chrom == chr
    #  tab.chr <- tab[idx.chr, ]
    #  Z.chr <- Z[idx.chr, ]
    #  zoomFN <- paste("zoom_chr", chr, ".RData", sep = "")
    #  save(tab.chr, Z.chr, resType, cbFN, genomeType, file = file.path(tDir, zoomFN))
   # }
    
    ## Define additional parameters before making results_calculations.html
    nredraw <- 0
    resultsDir <- tDir
    resultsFN <- "results.html"
    NewminLR <- minLR
    NewmaxLR <- maxLR
    NewThreshold <- Threshold
    tgzDir <- ""
    #save(nredraw,resultsDir,resultsFN,NewminLR,NewmaxLR,NewThreshold,file =file.path(tDir,"ExtraParams.RData"))
    ## Close log
     makeResultsHTML(tab,tempDir=tempDir,resultDir=resultDir,
     genomeType,nredraw,resultsDir,resultsFN,NewminLR,NewmaxLR,NewThreshold,Z,resType,methodNames,methodsIgnored,
     tabFNgzip, startTime, endTime, fsize)
    cat("----------------------------------------\n")
 
    
    
}

