`runCGHAnalysis` <-
function (tab, BioHMM =TRUE,UseCloneDists=TRUE,Lowess=TRUE, Lwidth=15,
Wavelet=TRUE, Wlevels=3,Runavg =TRUE,Rwidth=5,CBS =TRUE , alpha=0.05, Picard =TRUE, Km =20,S=-0.5,FusedLasso=TRUE,
fluv=FALSE,FDR=0.5,
rsm =FALSE, GLAD =TRUE, qlambda =0.999, FASeg=TRUE,sig=0.025, delta=0.1, srange = 50,
fineTune =FALSE,Quantreg =TRUE, lambda = 1,minLR=-2,maxLR = 2,Threshold =0.2,genomeType="HG18",tempDir=getwd(),resultDir="CGHResults")
{	
       #Check the parameters
   # Lowess <- Wavelet <- Quantreg <- Runavg <- CBS <- Picard <- BioHMM <- FusedLasso <- GLAD <- FASeg <- FALSE
    DirPath <- resultDir
    tDir <- file.path(tempDir, DirPath)
	# checking directories
    if(!file.exists(tempDir))
    	stop("The directory specified in 'tempDir' does not exist\n")
    if(file.exists(tDir))
    	stop("The directory name specified in 'resultDir' already exists\n")
    if(!dir.create(tDir))
    		stop(paste("Unable to create the output directory", tDir, "\n"))
    cat("----------------------------------------\n")
    cat("Validating parameters\n")
    
    ## Create a list of parameter settings specific to each algorithm
    ## so that we can easily use them in do.call() downstream.  This
    ## makes the analysis code easier to read and modify (e.g.,
    ## inclusion of additional array CGH algorithms)
    ## (Note: apList == "algorithm parameter list")

    apList <- list()
    fCall <- character(0)
    errormsgs <- character(0)

    ## Lowess
    if(Lowess != FALSE)  {
      if(is.na(Lwidth) | Lwidth > 50 | Lwidth < 5)
        errormsgs <- c(errormsgs, "Lowess/Lwidth")
      else  {
        apList <- c(apList, list(list(w = Lwidth)))
        fCall <- c(fCall, "calcLowess")
      }
    }

    ## Wavelet
    if(Wavelet != FALSE)  {
      if(is.na(Wlevels) | Wlevels < 1 | Wlevels > 10)
        errormsgs <- c(errormsgs, "Wavelet/Wlevels")
      else  {
        apList <- c(apList, list(list(n = Wlevels)))
        fCall <- c(fCall, "calcWavelet")
      }
    }

    ## Quantreg
    if(Quantreg != FALSE)  {
      if(is.na(lambda) | lambda > 10 | Lwidth < 0)
        errormsgs <- c(errormsgs, "Quantreg/lambda")
      else  {
        apList <- c(apList, list(list(lambda = lambda)))
        fCall <- c(fCall, "calcQuantreg")
      }
    }

    ## Runavg
    if(Runavg != FALSE)  {
      Rwidth <- as.numeric(Rwidth)
      if(is.na(Rwidth) | Rwidth > 50 | Rwidth < 5)
        errormsgs <- c(errormsgs, "Runavg/Rwidth")
      else  {
        apList <- c(apList, list(list(w = Rwidth)))
        fCall <- c(fCall, "calcRunavg")
      }
    }

    ## CBS
    if(CBS != FALSE)  {
      if(is.na(alpha) | alpha > 1 | alpha < 0)
        errormsgs <- c(errormsgs, "CBS/alpha")
      else
        apList <- c(apList, list(list(alpha = alpha)))
        fCall <- c(fCall, "calcCBS")
    }

    ## CGHseg (Picard)
    ## Note: there is no need to flip the sign for S with
    ##       the rewritten version of CGHseg
    if(Picard != FALSE)  {
      flag <- FALSE
      Km <- as.integer(Km)
      
      if(is.na(Km) | Km > 50 | Km < 5)  {
        errormsgs <- c(errormsgs, "Picard/Km")
        flag <- TRUE
      }
      if(is.na(S) | S >= 0 | S < -1)  {
        errormsgs <- c(errormsgs, "Picard/S")
        flag <- TRUE
      }
      if(!flag)  {
        apList <- c(apList, list(list(Km = Km, S = S)))
        fCall <- c(fCall, "calcPicard")
      }
    }

    ## BioHMM
    if(BioHMM != FALSE)  {
      useCloneDists <- UseCloneDists
      apList <- c(apList, list(list(useCloneDists = useCloneDists)))
      fCall <- c(fCall, "calcBioHMM")
    }

    ## FusedLasso
    if(FusedLasso != FALSE)  {
      flag <- FALSE

      if(!fluv)
        fdr <- NULL

      if(fluv)  {
        if(is.na(fdr) | fdr >= 1 | fdr <= 0)  {
          errormsgs <- c(errormsgs, "FusedLasso/fdr")
          flag <- TRUE
        }
      }

      if(!flag)  {
        apList <- c(apList, list(list(FDR = fdr, recalcSegmentMeans = rsm)))
        fCall <- c(fCall, "calcFL")
      }
    }

    ## GLAD
    if(GLAD != FALSE)  {
      if(is.na(qlambda) | qlambda < 0.9 | qlambda > 0.9999)
        errormsgs <- c(errormsgs, "GLAD/qlambda")
      else  {
        apList <- c(apList, list(list(qlambda = qlambda)))
        fCall <- c(fCall, "calcGLAD")
      }
    }

    ## FASeg
    if(FASeg != FALSE)  {
      flag <- FALSE
      if(is.na(sig) | sig > 1 | sig < 0)  {
        errormsgs <- c(errormsgs, "FASeg/sig")
        flag <- TRUE
      }
      if(is.na(delta) | delta > 0.5 | delta < 0.01)  {
        errormsgs <- c(errormsgs, "FASeg/delta")
        flag <- TRUE
      }
      if(!flag)  {
        apList <- c(apList, list(list(sig = sig, delta = delta, smooth.range = srange, fine.tune = fineTune)))
        fCall <- c(fCall, "calcFASeg")
      }
    }



    ## Name the top elements of the apList to their corresponding function calls
    names(apList) <- fCall

    if(is.na(Threshold) | Threshold > 0.5 | Threshold < 0.01)
      errormsgs <- c(errormsgs, "Other/Threshold")
    if(is.na(minLR) | minLR > -1 | minLR < -10)
      errormsgs <- c(errormsgs, "Other/minLR")
    if(is.na(maxLR) | maxLR > 10 | maxLR < 1)
      errormsgs <- c(errormsgs, "Other/maxLR")

    cat("Validation Done\n")
    

    if(length(errormsgs) == 0)  {
      ## Save list of parameter settings to unique temporary directory,
      ## close log file, and present a new HTML page for investigators
      ## to upload data
      #source(file.path(rScriptsDir, "checkUploadFileFormat.R"))
     	checkUploadFileFormat(tab,genomeType, tempDir,resultDir,Threshold,minLR,maxLR,apList)
       #run analysis
	   #source(file.path(rScriptsDir, "runAnalysis.R"))
#     runAnalysis(tempDir,resultDir)
     ## Make results_calculations.html by loading another R script
     #source(file.path(rScriptsDir, "makeResultsHTML.R"))
 #    makeResultsHTML(tab,tempDir=tempDir,resultDir=resultDir,genomeType)
 cat ("done")
     return (1)

      
    }else  {
      cat ("Problems with Input Parameters\n")
      cat("List of errors:\n")
      for(i in 1:length(errormsgs))
        cat( errormsgs[i],  "\n", sep = "")
       return (0)
    }

}

