`sm.quantreg` <-
function(y, lambda = 1.0, tau = 0.5, interpolate = FALSE)  {
  nona <- !is.na(y)

  ## Sparse matrices: E, D, and B
  if(!interpolate)  {
     y.nona <- y[nona]
    m <- length(y.nona)  *1.0
    E <- as.matrix.csr(0,m,m)
     diag(E) <- 1
     D <- diff(E)
      B <- rbind(E, lambda * D)
      ystar <- c(y.nona, rep(0,m-1))
    }else  {
    ## interpolate missing values via weighting
  
    y[!nona] <- 0
    m <- length(y) *1.0
    E <- as.matrix.csr(0,m,m)
  
    diag(E) <- 1
    D <- diff(E)
    tmp <- as.matrix.csr(0,m,m)
    diag(tmp) <- as.numeric(nona)
    B <- rbind(tmp, lambda * D)
    ystar <- c(y, rep(0,m-1))
  }

  ## use sparse matrices and accompanying mathematical operations
  
  try(myrq <- rq.fit.sfn(as.matrix.csr(B), ystar, tau))
  
  if(exists("myrq"))  {
    if(!interpolate)  {
      ys <- rep(NA, length(y))
      ys[nona] <- myrq$coef
    }else  {
      ys <- myrq$coef
    }
  }else  {
    cat("Error triggered within Quantreg algorithm!\n")
    ys <- NULL
  }
  return(ys)
}

