`sortChrs` <-
function(chrV)  {
  ## we divide up the chromosome string using "regular expressions"
  chr <- sub("^[chr]*([0-9xyXY]*)[p|q]*[A-Z]*[0-9.]*", "\\1", chrV)
  arm <- sub("^[chr]*[0-9xyXY]*([p|q])*[A-Z]*[0-9.]*", "\\1", chrV)
  ## cytoband1 is important for mouse chromosomes
  cytoband1 <- sub("^[chr]*[0-9xyXY]*[p|q]*([A-Z]*)[0-9.]*", "\\1", chrV)
  cytoband2 <- sub("^[chr]*[0-9xyXY]*[p|q]*[A-Z]*([0-9.]*)", "\\1", chrV)

  isMouse <- ifelse(any(cytoband1 != ""), TRUE, FALSE)
  
  ## The following line will throw a warning for non-numeric chromosomes
  ## because R does not know what number to assign for them and will
  ## instead assign NA for them
  tmp <- as.numeric(chr)
  not.numeric <- chr[is.na(tmp)]

  ## Assign numeric values for non-numeric chromosomes (e.g., X, Y)
  ## We assume non-numeric chromosomes go after numeric chromosomes
  ## and are sorted in alphabetical order
  if(length(not.numeric) > 0)  {
    max.numeric.chr <- max(tmp[!is.na(tmp)])
    not.numeric <- unique(sort(not.numeric))
    for(i in 1:length(not.numeric))
      tmp[chr == not.numeric[i]] <- max.numeric.chr + i
  }

  if(!isMouse)  {
    cytoband <- cytoband2
    if(any(cytoband != ""))  {
      ## The numbers in the cytogenetic band get smaller towards the centromere,
      ## which means the numbers in the "p" arm must be sorted in descending order,
      ## and the numbers in the "q" arm must be sorted in ascending order
      tmp.cytoband <- as.numeric(cytoband)
      tmp.cytoband[arm == "p"] <- -tmp.cytoband[arm == "p"]
      
      ## Sort by chromosome, then by arm, then by cytogenetic band
      res <- chrV[order(tmp, arm, tmp.cytoband)]
    }else if(any(arm != ""))  {
      ## Sort by chromosome, then by arm
      res <- chrV[order(tmp, arm)]
    }else  {
      ## Sort by chromosome
      res <- chrV[order(tmp)]
    }
  }else  {
    if(any(cytoband2 != ""))  {
      tmp.cytoband2 <- as.numeric(cytoband2)
      res <- chrV[order(tmp, arm, cytoband1, tmp.cytoband2)]
    }
  }
  
  return(res)
}

